package interpreter;

import java.text.MessageFormat;

import environment.RuntimeEnvironment;
import error.*;
import function.builtin.EVAL;
import parser.LispParser;
import sexpression.SExpression;

public class LispInterpreter {

    public static final String ANSI_RESET = "\u001B[0m";
    public static final String ANSI_GREEN = "\u001B[32m";

    protected RuntimeEnvironment environment;
    protected ErrorManager errorManager;
    private LispParser parser;

    public LispInterpreter() {
        this.environment = RuntimeEnvironment.getInstance();
        this.errorManager = this.environment.getErrorManager();
        this.parser = new LispParser(this.environment.getInput(), this.environment.getInputName());
    }

    public void interpret() {
        printGreeting();

        for (displayPrompt(); !parser.isEof(); displayPrompt())
            printValueOfNextSExpression();

        printFarewell();
    }

    protected void printGreeting() {}

    protected void displayPrompt() {}

    private void printValueOfNextSExpression() {
        try {
            printValueOfNextSExpressionWithException();
        } catch (LispException e) {
            erasePrompt();
            errorManager.generateError(e);
        }
    }

    private void printValueOfNextSExpressionWithException() {
        SExpression sExpression = parser.getNextSExpression();
        String result = MessageFormat.format("{0}{1}{2}", ANSI_GREEN, EVAL.eval(sExpression), ANSI_RESET);

        erasePrompt();
        environment.getOutput().println(result);
    }

    protected void erasePrompt() {}

    protected void printFarewell() {
        environment.getOutput().println();
    }

}
